@@ -31,7 +31,7 @@
 #include <glib/gi18n-lib.h>
 
 #ifdef HAVE_MEEGO
-#include <mx-gtk/mx-gtk.h>
+#include <mx/mx-gtk.h>
 #endif /* HAVE_MEEGO */
 
 #include <libempathy/empathy-utils.h>
@@ -60,7 +60,6 @@ typedef struct {
   GtkWidget *apply_button;
   GtkWidget *cancel_button;
   GtkWidget *entry_password;
-  GtkWidget *button_forget;
   GtkWidget *spinbutton_port;
   GtkWidget *enabled_checkbox;
   GtkWidget *radiobutton_reuse;
@@ -259,36 +258,6 @@ account_widget_checkbutton_toggled_cb (GtkWidget *widget,
   empathy_account_widget_changed (self);
 }
 
-static void
-account_widget_forget_clicked_cb (GtkWidget *button,
-    EmpathyAccountWidget *self)
-{
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  const gchar *param_name;
-
-  param_name = g_object_get_data (G_OBJECT (priv->entry_password),
-      "param_name");
-
-  DEBUG ("Unset %s", param_name);
-  empathy_account_settings_unset (priv->settings, param_name);
-  gtk_entry_set_text (GTK_ENTRY (priv->entry_password), "");
-
-  empathy_account_widget_changed (self);
-}
-
-static void
-account_widget_password_changed_cb (GtkWidget *entry,
-    EmpathyAccountWidget *self)
-{
-  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
-  const gchar *str;
-
-  str = gtk_entry_get_text (GTK_ENTRY (entry));
-  gtk_widget_set_sensitive (priv->button_forget, !EMP_STR_EMPTY (str));
-
-  priv->contains_pending_changes = TRUE;
-}
-
 static void
 account_widget_jabber_ssl_toggled_cb (GtkWidget *checkbutton_ssl,
     EmpathyAccountWidget *self)
@@ -355,6 +324,36 @@ account_widget_combobox_changed_cb (GtkWidget *widget,
   empathy_account_widget_changed (self);
 }
 
+static void
+clear_icon_released_cb (GtkEntry *entry,
+    GtkEntryIconPosition icon_pos,
+    GdkEvent *event,
+    EmpathyAccountWidget *self)
+{
+  EmpathyAccountWidgetPriv *priv = GET_PRIV (self);
+  const gchar *param_name;
+
+  param_name = g_object_get_data (G_OBJECT (entry), "param_name");
+
+  DEBUG ("Unset %s", param_name);
+  empathy_account_settings_unset (priv->settings, param_name);
+  gtk_entry_set_text (entry, "");
+
+  empathy_account_widget_changed (self);
+}
+
+static void
+password_entry_changed_cb (GtkEditable *entry,
+    EmpathyAccountWidget *self)
+{
+  const gchar *str;
+
+  str = gtk_entry_get_text (GTK_ENTRY (entry));
+
+  gtk_entry_set_icon_sensitive (GTK_ENTRY (entry),
+      GTK_ENTRY_ICON_SECONDARY, !EMP_STR_EMPTY (str));
+}
+
 void
 empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
     GtkWidget *widget,
@@ -419,6 +418,18 @@ empathy_account_widget_setup_widget (EmpathyAccountWidget *self,
       if (strstr (param_name, "password"))
         {
           gtk_entry_set_visibility (GTK_ENTRY (widget), FALSE);
+
+          /* Add 'clear' icon */
+          gtk_entry_set_icon_from_stock (GTK_ENTRY (widget),
+              GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_CLEAR);
+
+          gtk_entry_set_icon_sensitive (GTK_ENTRY (widget),
+              GTK_ENTRY_ICON_SECONDARY, !EMP_STR_EMPTY (str));
+
+          g_signal_connect (widget, "icon-release",
+              G_CALLBACK (clear_icon_released_cb), self);
+          g_signal_connect (widget, "changed",
+              G_CALLBACK (password_entry_changed_cb), self);
         }
 
       g_signal_connect (widget, "changed",
@@ -936,7 +947,6 @@ account_widget_build_msn (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-      self->ui_details->add_forget = TRUE;
     }
 }
 
@@ -1135,7 +1145,6 @@ account_widget_build_jabber (EmpathyAccountWidget *self,
         }
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-      self->ui_details->add_forget = TRUE;
       priv->spinbutton_port = spinbutton_port;
 
       g_signal_connect (checkbutton_ssl, "toggled",
@@ -1193,7 +1202,6 @@ account_widget_build_icq (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_uin");
-      self->ui_details->add_forget = TRUE;
     }
 }
 
@@ -1233,7 +1241,6 @@ account_widget_build_aim (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_screenname");
-      self->ui_details->add_forget = TRUE;
     }
 }
 
@@ -1275,7 +1282,6 @@ account_widget_build_yahoo (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-      self->ui_details->add_forget = TRUE;
     }
 }
 
@@ -1313,7 +1319,6 @@ account_widget_build_groupwise (EmpathyAccountWidget *self,
           NULL);
 
       self->ui_details->default_focus = g_strdup ("entry_id");
-      self->ui_details->add_forget = TRUE;
     }
 }
 
@@ -1356,7 +1361,7 @@ account_widget_switch_flipped_cb (MxGtkLightSwitch *sw,
     gboolean state,
     gpointer user_data)
 #else
-account_widget_enabled_toggled_cb (GtkToggleButton *toggle_button,
+account_widget_enabled_released_cb (GtkToggleButton *toggle_button,
     gpointer user_data)
 #endif /* HAVE_MEEGO */
 {
@@ -1582,8 +1587,8 @@ add_enable_checkbox (EmpathyAccountWidget *self,
   g_signal_connect (G_OBJECT (priv->enabled_checkbox), "switch-flipped",
       G_CALLBACK (account_widget_switch_flipped_cb), self);
 #else
-  g_signal_connect (G_OBJECT (priv->enabled_checkbox), "toggled",
-      G_CALLBACK (account_widget_enabled_toggled_cb), self);
+  g_signal_connect (G_OBJECT (priv->enabled_checkbox), "released",
+      G_CALLBACK (account_widget_enabled_released_cb), self);
 #endif /* HAVE_MEEGO */
 }
 
@@ -1695,29 +1700,6 @@ do_constructed (GObject *obj)
           NULL);
     }
 
-  /* handle forget button */
-  if (self->ui_details->add_forget)
-    {
-      const gchar *password = NULL;
-
-      priv->button_forget = GTK_WIDGET (gtk_builder_get_object
-          (self->ui_details->gui, "button_forget"));
-      priv->entry_password = GTK_WIDGET (gtk_builder_get_object
-          (self->ui_details->gui, "entry_password"));
-
-      password = empathy_account_settings_get_string (priv->settings,
-          "password");
-      gtk_widget_set_sensitive (priv->button_forget,
-          !EMP_STR_EMPTY (password));
-
-      g_signal_connect (priv->button_forget, "clicked",
-          G_CALLBACK (account_widget_forget_clicked_cb),
-          self);
-      g_signal_connect (priv->entry_password, "changed",
-          G_CALLBACK (account_widget_password_changed_cb),
-          self);
-    }
-
   /* dup and init the account-manager */
   priv->account_manager = tp_account_manager_dup ();
 
@@ -1736,9 +1718,9 @@ do_constructed (GObject *obj)
 
       /* We'll change this button to a "Log in" one if we are creating a new
        * account and are connected. */
-      empathy_signal_connect_weak (priv->account_manager,
+      tp_g_signal_connect_object (priv->account_manager,
           "most-available-presence-changed",
-          G_CALLBACK (presence_changed_cb), obj);
+          G_CALLBACK (presence_changed_cb), obj, 0);
 
       gtk_box_pack_end (GTK_BOX (hbox), priv->apply_button, TRUE,
           TRUE, 3);
