@@ -871,6 +871,32 @@ widget_avatar_button_press_event_cb (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+set_avatar_cb (GObject *source,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GError *error = NULL;
+
+  if (!tp_account_set_avatar_finish (TP_ACCOUNT (source), res, &error)) {
+      DEBUG ("Failed to set Account.Avatar: %s", error->message);
+      g_error_free (error);
+  }
+}
+
+static void
+set_avatar_on_account (TpAccount *account,
+    const gchar *data,
+    gsize size,
+    const gchar *mime_type)
+{
+  DEBUG ("%s Account.Avatar on %s", size > 0 ? "Set": "Clear",
+      tp_proxy_get_object_path (account));
+
+  tp_account_set_avatar_async (account, (const guchar *) data, size,
+      mime_type, set_avatar_cb, NULL);
+}
+
 static void
 contact_widget_avatar_changed_cb (EmpathyAvatarChooser *chooser,
                                   EmpathyContactWidget *information)
@@ -878,12 +904,37 @@ contact_widget_avatar_changed_cb (EmpathyAvatarChooser *chooser,
   const gchar *data;
   gsize size;
   const gchar *mime_type;
+  TpAccount *account;
 
   empathy_avatar_chooser_get_image_data (
       EMPATHY_AVATAR_CHOOSER (information->widget_avatar),
       &data, &size, &mime_type);
-  empathy_tp_contact_factory_set_avatar (information->factory,
-      data, size, mime_type);
+
+  account = empathy_contact_get_account (information->contact);
+  set_avatar_on_account (account, data, size, mime_type);
+}
+
+static void
+set_nickname_cb (GObject *source,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GError *error = NULL;
+
+  if (!tp_account_set_nickname_finish (TP_ACCOUNT (source), res, &error))
+    {
+      DEBUG ("Failed to set Account.Nickname: %s", error->message);
+      g_error_free (error);
+    }
+}
+
+static void
+set_alias_on_account (TpAccount *account,
+    const gchar *alias)
+{
+  DEBUG ("Set Account.Nickname to %s", alias);
+
+  tp_account_set_nickname_async (account, alias, set_nickname_cb, NULL);
 }
 
 static gboolean
@@ -896,8 +947,19 @@ contact_widget_entry_alias_focus_event_cb (GtkEditable *editable,
       const gchar *alias;
 
       alias = gtk_entry_get_text (GTK_ENTRY (editable));
-      empathy_tp_contact_factory_set_alias (information->factory,
-          information->contact, alias);
+
+      if (empathy_contact_is_user (information->contact))
+        {
+          TpAccount * account;
+
+          account = empathy_contact_get_account (information->contact);
+          set_alias_on_account (account, alias);
+        }
+      else
+        {
+          empathy_tp_contact_factory_set_alias (information->factory,
+              information->contact, alias);
+        }
     }
 
   return FALSE;
@@ -907,10 +969,13 @@ static void
 update_avatar_chooser_account_cb (EmpathyAccountChooser *account_chooser,
                                   EmpathyAvatarChooser *avatar_chooser)
 {
-  TpConnection *connection;
+  TpAccount *account;
 
-  connection = empathy_account_chooser_get_connection (account_chooser);
-  g_object_set (avatar_chooser, "connection", connection, NULL);
+  account = empathy_account_chooser_get_account (account_chooser);
+  if (account == NULL)
+    return;
+
+  empathy_avatar_chooser_set_account (avatar_chooser, account);
 }
 
 static void
