@@ -38,7 +38,6 @@
 
 #include "libempathy/empathy-enum-types.h"
 #include "libempathy/empathy-location.h"
-#include "libempathy/empathy-tp-contact-factory.h"
 #include "libempathy/empathy-utils.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_LOCATION
@@ -182,6 +181,16 @@ empathy_location_manager_class_init (EmpathyLocationManagerClass *class)
   g_type_class_add_private (object_class, sizeof (EmpathyLocationManagerPriv));
 }
 
+static void
+publish_location_cb (TpConnection *connection,
+                     const GError *error,
+                     gpointer user_data,
+                     GObject *weak_object)
+{
+  if (error != NULL)
+      DEBUG ("Error setting location: %s", error->message);
+}
+
 static void
 publish_location (EmpathyLocationManager *self,
     TpConnection *conn,
@@ -191,7 +200,6 @@ publish_location (EmpathyLocationManager *self,
   guint connection_status = -1;
   gboolean can_publish;
   EmpathyConf *conf = empathy_conf_get ();
-  EmpathyTpContactFactory *factory;
 
   if (!conn)
     return;
@@ -215,9 +223,8 @@ publish_location (EmpathyLocationManager *self,
       (g_hash_table_size (priv->location) == 0 ? "empty" : ""),
       conn);
 
-  factory = empathy_tp_contact_factory_dup_singleton (conn);
-  empathy_tp_contact_factory_set_location (factory, priv->location);
-  g_object_unref (factory);
+  tp_cli_connection_interface_location_call_set_location (conn, -1,
+      priv->location, publish_location_cb, NULL, NULL, G_OBJECT (self));
 }
 
 typedef struct
@@ -499,24 +506,6 @@ initial_position_cb (GeocluePosition *position,
     }
 }
 
-static gboolean
-set_requirements (EmpathyLocationManager *self)
-{
-  EmpathyLocationManagerPriv *priv = GET_PRIV (self);
-  GError *error = NULL;
-
-  if (!geoclue_master_client_set_requirements (priv->gc_client,
-          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, FALSE, priv->resources,
-          &error))
-    {
-      DEBUG ("set_requirements failed: %s", error->message);
-      g_error_free (error);
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 static void
 update_resources (EmpathyLocationManager *self)
 {
@@ -530,8 +519,13 @@ update_resources (EmpathyLocationManager *self)
   /* As per Geoclue bug #15126, using NONE results in no address
    * being found as geoclue-manual report an empty address with
    * accuracy = NONE */
-  if (!set_requirements (self))
-    return;
+  if (!geoclue_master_client_set_requirements (priv->gc_client,
+          GEOCLUE_ACCURACY_LEVEL_COUNTRY, 0, TRUE, priv->resources,
+          NULL))
+    {
+      DEBUG ("set_requirements failed");
+      return;
+    }
 
   geoclue_address_get_address_async (priv->gc_address,
       initial_address_cb, self);
@@ -559,8 +553,7 @@ setup_geoclue (EmpathyLocationManager *self)
       return;
     }
 
-  if (!set_requirements (self))
-    return;
+  update_resources (self);
 
   /* Get updated when the position is changes */
   priv->gc_position = geoclue_master_client_create_position (
@@ -708,8 +701,8 @@ account_manager_prepared_cb (GObject *source_object,
     {
       TpAccount *account = TP_ACCOUNT (l->data);
 
-      empathy_signal_connect_weak (account, "status-changed",
-          G_CALLBACK (new_connection_cb), G_OBJECT (self));
+      tp_g_signal_connect_object (account, "status-changed",
+          G_CALLBACK (new_connection_cb), self, 0);
     }
   g_list_free (accounts);
 }
