@@ -141,6 +141,7 @@ typedef struct
 
   /* Properties to pass to the channel when requesting it */
   GHashTable *request;
+  gint64 timestamp;
   EmpathyDispatcherRequestCb *cb;
   gpointer user_data;
   gpointer *request_data;
@@ -218,6 +219,7 @@ new_dispatcher_request_data (EmpathyDispatcher *self,
                              guint handle_type,
                              guint handle,
                              GHashTable *request,
+                             gint64 timestamp,
                              EmpathyContact *contact,
                              EmpathyDispatcherRequestCb *cb,
                              gpointer user_data)
@@ -233,6 +235,7 @@ new_dispatcher_request_data (EmpathyDispatcher *self,
   result->handle_type = handle_type;
   result->handle = handle;
   result->request = request;
+  result->timestamp = timestamp;
 
   if (contact != NULL)
     result->contact = g_object_ref (contact);
@@ -412,8 +415,6 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   connection = tp_channel_borrow_connection (TP_CHANNEL (proxy));
 
-  priv->channels = g_list_remove (priv->channels, proxy);
-
   cd = g_hash_table_lookup (priv->connections, connection);
   /* Connection itself invalidated? */
   if (cd == NULL)
@@ -425,6 +426,8 @@ dispatcher_channel_invalidated_cb (TpProxy *proxy,
 
   g_hash_table_remove (cd->dispatched_channels, object_path);
   g_hash_table_remove (cd->dispatching_channels, object_path);
+
+  priv->channels = g_list_remove (priv->channels, proxy);
 }
 
 static void
@@ -570,7 +573,8 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
                                    guint handle,
                                    GHashTable *properties,
                                    gboolean incoming,
-                                   const GPtrArray *requests_satisfied)
+                                   const GPtrArray *requests_satisfied,
+                                   gint64 timestamp)
 {
   EmpathyDispatcherPriv *priv = GET_PRIV (self);
   TpChannel         *channel;
@@ -627,7 +631,7 @@ dispatcher_connection_new_channel (EmpathyDispatcher *self,
   priv->channels = g_list_prepend (priv->channels, channel);
 
   operation = empathy_dispatch_operation_new (connection, channel, NULL,
-    incoming);
+    incoming, timestamp);
 
   g_object_unref (channel);
 
@@ -688,7 +692,8 @@ dispatcher_connection_new_channel_with_properties (
     TpConnection *connection,
     const gchar *object_path,
     GHashTable *properties,
-    const GPtrArray *requests_satisfied)
+    const GPtrArray *requests_satisfied,
+    gint64 timestamp)
 {
   const gchar *channel_type;
   guint handle_type;
@@ -732,7 +737,7 @@ dispatcher_connection_new_channel_with_properties (
 
   dispatcher_connection_new_channel (self, connection,
     object_path, channel_type, handle_type, handle, properties, !requested,
-    requests_satisfied);
+    requests_satisfied, timestamp);
 }
 
 static void
@@ -819,14 +824,9 @@ connection_ready_cb (TpConnection *connection,
       goto out;
     }
 
-  if (tp_proxy_has_interface_by_id (TP_PROXY (connection),
-      TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
-    {
-      tp_cli_dbus_properties_call_get_all (connection, -1,
-        TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
-        dispatcher_connection_got_all,
-        NULL, NULL, G_OBJECT (self));
-    }
+  tp_cli_dbus_properties_call_get_all (connection, -1,
+      TP_IFACE_CONNECTION_INTERFACE_REQUESTS, dispatcher_connection_got_all,
+      NULL, NULL, G_OBJECT (self));
 
   /* Advertise VoIP capabilities */
   capabilities = g_ptr_array_sized_new (1);
@@ -1219,9 +1219,9 @@ empathy_dispatcher_init (EmpathyDispatcher *self)
   tp_account_manager_prepare_async (priv->account_manager, NULL,
       account_manager_prepared_cb, self);
 
-  empathy_signal_connect_weak (priv->account_manager,
+  tp_g_signal_connect_object (priv->account_manager,
       "account-validity-changed", G_CALLBACK (account_validity_changed_cb),
-      G_OBJECT (self));
+      self, 0);
 
   priv->request_channel_class_async_ids = g_hash_table_new (g_direct_hash,
     g_direct_equal);
@@ -1281,155 +1281,29 @@ dispatcher_request_failed (EmpathyDispatcher *self,
   free_dispatcher_request_data (request_data);
 }
 
-static void
-dispatcher_connection_new_requested_channel (EmpathyDispatcher *self,
-  DispatcherRequestData *request_data,
-  const gchar *object_path,
-  GHashTable *properties,
-  const GError *error)
-{
-  EmpathyDispatcherPriv *priv = GET_PRIV (self);
-  EmpathyDispatchOperation *operation = NULL;
-  ConnectionData *conn_data;
-
-  /* The DispatcherRequestData owns a ref on the self object. As the request
-   * data could be destroyed (when calling dispatcher_request_failed for
-   * example) we keep a ref on self to be sure it stays alive while we are
-   * executing this function. */
-  g_object_ref (self);
-
-  conn_data = g_hash_table_lookup (priv->connections,
-    request_data->connection);
-
-  if (error)
-    {
-      DEBUG ("Channel request failed: %s", error->message);
-
-      dispatcher_request_failed (self, request_data, error);
-
-      return;
-    }
-
-  operation = g_hash_table_lookup (conn_data->dispatching_channels,
-        object_path);
-
-  if (operation == NULL)
-    {
-      DispatchData *data = g_hash_table_lookup (conn_data->dispatched_channels,
-        object_path);
-
-      if (data != NULL)
-        {
-          operation = empathy_dispatch_operation_new_with_wrapper (
-            request_data->connection,
-            data->channel, request_data->contact, FALSE,
-            data->channel_wrapper);
-        }
-      else
-        {
-          TpChannel *channel;
-
-          if (properties != NULL)
-            channel = tp_channel_new_from_properties (request_data->connection,
-              object_path, properties, NULL);
-          else
-            channel = tp_channel_new (request_data->connection, object_path,
-              request_data->channel_type, request_data->handle_type,
-              request_data->handle, NULL);
-
-          g_signal_connect (channel, "invalidated",
-            G_CALLBACK (dispatcher_channel_invalidated_cb),
-            request_data->dispatcher);
-
-          priv->channels = g_list_prepend (priv->channels, channel);
-
-          operation = empathy_dispatch_operation_new (request_data->connection,
-             channel, request_data->contact, FALSE);
-          g_object_unref (channel);
-        }
-    }
-  else
-    {
-      /* Already existed set potential extra information */
-      g_object_set (G_OBJECT (operation),
-        "contact", request_data->contact,
-        NULL);
-    }
-
-  request_data->operation = operation;
-
-  /* (pre)-approve this right away as we requested it
-   * This might cause the channel to be claimed, in which case the operation
-   * will disappear. So ref it, and check the status before starting the
-   * dispatching */
-
-  g_object_ref (operation);
-  empathy_dispatch_operation_approve (operation);
-
-   if (empathy_dispatch_operation_get_status (operation) <
-     EMPATHY_DISPATCHER_OPERATION_STATE_APPROVING)
-      dispatcher_start_dispatching (request_data->dispatcher, operation,
-          conn_data);
-
-  g_object_unref (operation);
-}
-
-static void
-dispatcher_request_channel_cb (TpConnection *connection,
-                               const gchar  *object_path,
-                               const GError *error,
-                               gpointer user_data,
-                               GObject *weak_object)
-{
-  DispatcherRequestData *request_data = (DispatcherRequestData *) user_data;
-  EmpathyDispatcher *self =
-      EMPATHY_DISPATCHER (request_data->dispatcher);
-
-  request_data->pending_call = NULL;
-
-  dispatcher_connection_new_requested_channel (self,
-    request_data, object_path, NULL, error);
-}
-
 static void
 dispatcher_request_channel (DispatcherRequestData *request_data)
 {
-  if (tp_proxy_has_interface_by_id (TP_PROXY (request_data->connection),
-      TP_IFACE_QUARK_CONNECTION_INTERFACE_REQUESTS))
-    {
-      /* Extend the request_data to be a valid request */
-      g_assert (request_data->request == NULL);
-      request_data->request = tp_asv_new (
-        TP_IFACE_CHANNEL ".ChannelType",
-          G_TYPE_STRING, request_data->channel_type,
-        TP_IFACE_CHANNEL ".TargetHandleType",
-          G_TYPE_UINT, request_data->handle_type,
-        NULL);
+  /* Extend the request_data to be a valid request */
+  g_assert (request_data->request == NULL);
+  request_data->request = tp_asv_new (
+      TP_IFACE_CHANNEL ".ChannelType",
+        G_TYPE_STRING, request_data->channel_type,
+      TP_IFACE_CHANNEL ".TargetHandleType",
+        G_TYPE_UINT, request_data->handle_type,
+      NULL);
 
-      if (request_data->handle_type != TP_HANDLE_TYPE_NONE)
-        tp_asv_set_uint32 (request_data->request,
-          TP_IFACE_CHANNEL ".TargetHandle", request_data->handle);
+  if (request_data->handle_type != TP_HANDLE_TYPE_NONE)
+    tp_asv_set_uint32 (request_data->request, TP_IFACE_CHANNEL ".TargetHandle",
+        request_data->handle);
 
-      empathy_dispatcher_call_create_or_ensure_channel (
-        request_data->dispatcher, request_data);
-    }
-  else
-    {
-      TpProxyPendingCall *call = tp_cli_connection_call_request_channel (
-        request_data->connection, -1,
-        request_data->channel_type,
-        request_data->handle_type,
-        request_data->handle,
-        TRUE, dispatcher_request_channel_cb,
-        request_data, NULL, NULL);
-
-      if (call != NULL)
-        request_data->pending_call = call;
-    }
+  empathy_dispatcher_call_create_or_ensure_channel (request_data->dispatcher,
+      request_data);
 }
 
 void
 empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
+                                      gint64 timestamp,
                                       EmpathyDispatcherRequestCb *callback,
                                       gpointer user_data)
 {
@@ -1461,7 +1335,8 @@ empathy_dispatcher_chat_with_contact (EmpathyContact *contact,
   /* The contact handle might not be known yet */
   request_data = new_dispatcher_request_data (self, connection,
     TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_CONTACT,
-    empathy_contact_get_handle (contact), NULL, contact, callback, user_data);
+    empathy_contact_get_handle (contact), NULL, timestamp, contact,
+    callback, user_data);
   request_data->should_ensure = TRUE;
 
   connection_data->outstanding_requests = g_list_prepend
@@ -1478,6 +1353,7 @@ typedef struct
   EmpathyDispatcher *dispatcher;
   EmpathyDispatcherRequestCb *callback;
   gpointer user_data;
+  gint64 timestamp;
 } ChatWithContactIdData;
 
 static void
@@ -1500,8 +1376,8 @@ dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
     }
   else
     {
-      empathy_dispatcher_chat_with_contact (contact, data->callback,
-          data->user_data);
+      empathy_dispatcher_chat_with_contact (contact, data->timestamp,
+          data->callback, data->user_data);
     }
 
   g_object_unref (data->dispatcher);
@@ -1511,6 +1387,7 @@ dispatcher_chat_with_contact_id_cb (EmpathyTpContactFactory *factory,
 void
 empathy_dispatcher_chat_with_contact_id (TpConnection *connection,
                                          const gchar *contact_id,
+                                         gint64 timestamp,
                                          EmpathyDispatcherRequestCb *callback,
                                          gpointer user_data)
 {
@@ -1527,6 +1404,7 @@ empathy_dispatcher_chat_with_contact_id (TpConnection *connection,
   data->dispatcher = self;
   data->callback = callback;
   data->user_data = user_data;
+  data->timestamp = timestamp;
   empathy_tp_contact_factory_get_from_id (factory, contact_id,
       dispatcher_chat_with_contact_id_cb, data, NULL, NULL);
 
@@ -1569,6 +1447,7 @@ dispatcher_request_handles_cb (TpConnection *connection,
 void
 empathy_dispatcher_join_muc (TpConnection *connection,
                              const gchar *roomname,
+                             gint64 timestamp,
                              EmpathyDispatcherRequestCb *callback,
                              gpointer user_data)
 {
@@ -1590,7 +1469,7 @@ empathy_dispatcher_join_muc (TpConnection *connection,
 
   /* Don't know the room handle yet */
   request_data  = new_dispatcher_request_data (self, connection,
-    TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM, 0, NULL,
+    TP_IFACE_CHANNEL_TYPE_TEXT, TP_HANDLE_TYPE_ROOM, 0, NULL, timestamp,
     NULL, callback, user_data);
   request_data->should_ensure = TRUE;
 
@@ -1751,7 +1630,7 @@ empathy_dispatcher_call_create_or_ensure_channel (
       call = tp_cli_channel_dispatcher_call_ensure_channel (
           priv->channel_dispatcher,
           -1, tp_proxy_get_object_path (TP_PROXY (account)),
-          request_data->request, 0, handler,
+          request_data->request, request_data->timestamp, handler,
           dispatcher_create_channel_cb, request_data, NULL, NULL);
     }
   else
@@ -1759,7 +1638,7 @@ empathy_dispatcher_call_create_or_ensure_channel (
       call = tp_cli_channel_dispatcher_call_create_channel (
           priv->channel_dispatcher,
           -1, tp_proxy_get_object_path (TP_PROXY (account)),
-          request_data->request, 0, handler,
+          request_data->request, request_data->timestamp, handler,
           dispatcher_create_channel_cb, request_data, NULL,
           G_OBJECT (dispatcher));
     }
@@ -1785,6 +1664,7 @@ void
 empathy_dispatcher_create_channel (EmpathyDispatcher *self,
                                    TpConnection *connection,
                                    GHashTable *request,
+                                   gint64 timestamp,
                                    EmpathyDispatcherRequestCb *callback,
                                    gpointer user_data)
 {
@@ -1813,7 +1693,7 @@ empathy_dispatcher_create_channel (EmpathyDispatcher *self,
   handle = tp_asv_get_uint32 (request, TP_IFACE_CHANNEL ".TargetHandle", NULL);
 
   request_data  = new_dispatcher_request_data (self, connection,
-    channel_type, handle_type, handle, request,
+    channel_type, handle_type, handle, request, timestamp,
     NULL, callback, user_data);
 
   connection_data->outstanding_requests = g_list_prepend
@@ -2200,7 +2080,7 @@ empathy_dispatcher_handle_channels (EmpathyHandler *handler,
       properties = g_value_get_boxed (g_value_array_get_nth (arr, 1));
 
       dispatcher_connection_new_channel_with_properties (self,
-        connection, object_path, properties, requests_satisfied);
+        connection, object_path, properties, requests_satisfied, timestamp);
     }
 
   return TRUE;
