@@ -20,7 +20,8 @@
 
 /* empathy-account-assistant.c */
 
-#include <glib/gi18n.h>
+#include <config.h>
+#include <glib/gi18n-lib.h>
 #include <telepathy-glib/util.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -901,11 +902,13 @@ impl_signal_apply (GtkAssistant *assistant)
             FALSE);
       return;
     }
-  else if (current_page == PAGE_ENTER_CREATE &&
+  else if (current_page >= PAGE_ENTER_CREATE &&
       priv->settings != NULL &&
       empathy_account_settings_is_valid (priv->settings))
     {
       account_assistant_apply_account_and_finish (self, priv->settings, TRUE);
+      g_object_unref (priv->settings);
+      priv->settings = NULL;
     }
 }
 
